# coding: utf-8
"""
Process a weather forecast json file to plot the time evolution of temperature
of a given day in a given city
"""
from urllib.request import urlopen
import json
import numpy
import matplotlib.pyplot as plt
import matplotlib.image as mpimg
from io import BytesIO


def plot_day_tempe(day_key=0, city_name='strasbourg'):
    """Plot the time evolution of predicted temperature"""

    jsonfile_url = f"https://www.prevision-meteo.ch/services/json/{city_name}"

    try:
        f = urlopen(jsonfile_url, timeout=10)  # open url
    except Exception:
        print("Le téléchargement a échoué : on lit une version locale.")
        f = open("exos/Strasbourg.json")

    jsondict = json.loads(f.read())  # Read JSON file
    # Raise error if city does not exist
    if 'errors' in jsondict:
        print(
            f"{jsondict['errors'][0]['text']}: {jsondict['errors'][0]['description']}")
        exit(1)
    try:
        day = jsondict[f'fcst_day_{day_key:d}']
    except KeyError:
        # Raise error if day key does not exist
        day_keys = [int(fcst_key[-1])
                    for fcst_key in jsondict.keys()
                    if fcst_key.startswith('fcst_day_')]
        print(f"Error: day_key must be in {day_keys}")
        exit(1)
    day_hd = day['hourly_data']

    tempe = []

    for hour, data in day_hd.items():
        tempe.append([int(hour[:-3]), data['TMP2m']])

    tempe.sort()
    t = numpy.array(tempe)
    t = t.transpose()

    # Plot T = T(hour)
    fig, ax = plt.subplots()  # initialise figure and plot area
    title = f"{day['day_long']} {day['date']} à {city_name.title()}"
    fig.suptitle(title, fontsize=14, fontweight='bold')
    fig.subplots_adjust(top=0.85)

    ax.set_title('Évolution horaire de la température')
    ax.set_xlabel('Heure [h]')
    ax.set_ylabel('Température [dég. C]')

    ax.plot(t[0], t[1])  # plot t[1] (tempe) as a function of t[0] (hour)

    # Add meteo icon to plot
    icon = BytesIO(urlopen(day['icon_big']).read())  # Open weather icon

    axicon = fig.add_axes([0.8, 0.8, 0.15, 0.15])
    img = mpimg.imread(icon)  # initialise image
    axicon.set_xticks([])  # Remove axes ticks
    axicon.set_yticks([])
    axicon.imshow(img)  # trigger the image show

    plt.show()  # trigger the figure display


if __name__ == '__main__':
    # Create a CLI using https://github.com/google/python-fire
    import fire
    fire.Fire(plot_day_tempe)
